@@ -104,13 +104,18 @@ int main(int argc, char **argv)
     return 2;
 
 #ifdef UNDER_CE
+  /* !checksrc! disable BANNEDFUNC 1 */
   stat(file, &file_info);
 #else
   fstat(fileno(fp), &file_info);
 #endif
 
   /* In Windows, this inits the Winsock stuff */
-  curl_global_init(CURL_GLOBAL_ALL);
+  res = curl_global_init(CURL_GLOBAL_ALL);
+  if(res) {
+    fclose(fp);
+    return (int)res;
+  }
 
   /* get a curl handle */
   curl = curl_easy_init();
@@ -160,5 +165,5 @@ int main(int argc, char **argv)
   fclose(fp); /* close the local file */
 
   curl_global_cleanup();
-  return 0;
+  return (int)res;
 }
