@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2022, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -30,6 +30,14 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#ifdef _WIN32
+#undef stat
+#define stat _stat
+#undef fstat
+#define fstat _fstat
+#define fileno _fileno
+#endif
+
 int main(void)
 {
   CURL *curl;
@@ -38,13 +46,27 @@ int main(void)
   curl_off_t speed_upload, total_time;
   FILE *fd;
 
+  res = curl_global_init(CURL_GLOBAL_ALL);
+  if(res)
+    return (int)res;
+
   fd = fopen("debugit", "rb"); /* open file to upload */
-  if(!fd)
+  if(!fd) {
+    curl_global_cleanup();
     return 1; /* cannot continue */
+  }
 
   /* to get the file size */
-  if(fstat(fileno(fd), &file_info) != 0)
+#ifdef UNDER_CE
+  /* !checksrc! disable BANNEDFUNC 1 */
+  if(stat("debugit", &file_info) != 0) {
+#else
+  if(fstat(fileno(fd), &file_info) != 0) {
+#endif
+    fclose(fd);
+    curl_global_cleanup();
     return 1; /* cannot continue */
+  }
 
   curl = curl_easy_init();
   if(curl) {
@@ -85,5 +107,6 @@ int main(void)
     curl_easy_cleanup(curl);
   }
   fclose(fd);
+  curl_global_cleanup();
   return 0;
 }
