@@ -36,69 +36,62 @@
  */
 int main(void)
 {
-  CURL *curl;
-  CURL *curl2;
+  CURL *http_handle;
+  CURL *http_handle2;
+  CURLM *multi_handle;
+
+  int still_running = 1; /* keep number of running handles */
 
   CURLcode res = curl_global_init(CURL_GLOBAL_ALL);
   if(res)
     return (int)res;
 
-  curl = curl_easy_init();
-  curl2 = curl_easy_init();
+  http_handle = curl_easy_init();
+  http_handle2 = curl_easy_init();
 
-  if(curl && curl2) {
+  /* set options */
+  curl_easy_setopt(http_handle, CURLOPT_URL, "https://www.example.com/");
 
-    CURLM *multi;
+  /* set options */
+  curl_easy_setopt(http_handle2, CURLOPT_URL, "http://localhost/");
 
-    /* set options */
-    curl_easy_setopt(curl, CURLOPT_URL, "https://www.example.com/");
+  /* init a multi stack */
+  multi_handle = curl_multi_init();
 
-    /* set options */
-    curl_easy_setopt(curl2, CURLOPT_URL, "http://localhost/");
+  /* add the individual transfers */
+  curl_multi_add_handle(multi_handle, http_handle);
+  curl_multi_add_handle(multi_handle, http_handle2);
 
-    /* init a multi stack */
-    multi = curl_multi_init();
-    if(multi) {
+  while(still_running) {
+    CURLMsg *msg;
+    int queued;
+    CURLMcode mc = curl_multi_perform(multi_handle, &still_running);
 
-      int still_running = 1; /* keep number of running handles */
+    if(still_running)
+      /* wait for activity, timeout or "nothing" */
+      mc = curl_multi_poll(multi_handle, NULL, 0, 1000, NULL);
 
-      /* add the individual transfers */
-      curl_multi_add_handle(multi, curl);
-      curl_multi_add_handle(multi, curl2);
+    if(mc)
+      break;
 
-      while(still_running) {
-        CURLMsg *msg;
-        int queued;
-
-        CURLMcode mc = curl_multi_perform(multi, &still_running);
-
-        if(still_running)
-          /* wait for activity, timeout or "nothing" */
-          mc = curl_multi_poll(multi, NULL, 0, 1000, NULL);
-
-        if(mc)
-          break;
-
-        do {
-          msg = curl_multi_info_read(multi, &queued);
-          if(msg) {
-            if(msg->msg == CURLMSG_DONE) {
-              /* a transfer ended */
-              fprintf(stderr, "Transfer completed\n");
-            }
-          }
-        } while(msg);
+    do {
+      msg = curl_multi_info_read(multi_handle, &queued);
+      if(msg) {
+        if(msg->msg == CURLMSG_DONE) {
+          /* a transfer ended */
+          fprintf(stderr, "Transfer completed\n");
+        }
       }
-
-      curl_multi_remove_handle(multi, curl);
-      curl_multi_remove_handle(multi, curl2);
-
-      curl_multi_cleanup(multi);
-    }
+    } while(msg);
   }
 
-  curl_easy_cleanup(curl);
-  curl_easy_cleanup(curl2);
+  curl_multi_remove_handle(multi_handle, http_handle);
+  curl_multi_remove_handle(multi_handle, http_handle2);
+
+  curl_multi_cleanup(multi_handle);
+
+  curl_easy_cleanup(http_handle);
+  curl_easy_cleanup(http_handle2);
 
   curl_global_cleanup();
 
